// 
// Glow Hott pour télémesure Graupner Hott
// 
// Gestion messagerie Hott
//
// Auteur : www.thyzoon.fr 
//

#define LEDSECU_OFF           PORTD &= ~(1<<4);
#define LEDSECU_ON            PORTD |= (1<<4);

#include "Message.h"
#include "Mesure.h"
#include <EEPROM.h>

#define VERSION         "1.18"

// -------------------------------------
// Valeurs par défaut stockées en EEPROM
//
#define RALENTI_GAZ	1300      // en µs
#define COUPURE_GAZ 	1080  // en µs
#define POS_ARMEMENT  	1800  // en µs
#define NB_PALES        2
#define DELAI_CHAUFFAGE 1500  // en ms
#define COURANT_DEM     100   // en % courant moteur à l'arrêt pour démarrage
#define COURANT_TRN     60    // en % courant moteur tournant
#define VITESSE_GAZ     60    // 60 µS pour 20 ms (20ms = période impulsion RX)
#define REG_MOT_MAX     1300  // Regime moteur maxi. pour limitation du compte-tours (/10)
#define ALARME_MOTEUR   480   //  480s = 8 minutes
#define ALARME_REPET    15    // 15s
#define ALARME_TEMPERATURE 100 // 100°C
#define ALARME_ON       0     // OFF   
#define FRANCAIS        0     // 0 = Francais
#define ANGLAIS         1     // 1 = Anglais

// buffer message
static uint8_t _hott_serial_buffer[173];   //création d'une variable tampon pour stocker les struct

// pointer des structures vers le tampon "_hott_serial_buffer"
struct HOTT_GAM_MSG         *hott_gam_msg = (struct HOTT_GAM_MSG *)&_hott_serial_buffer[0];
struct HOTT_TEXTMODE_MSG	*hott_txt_msg =	(struct HOTT_TEXTMODE_MSG *)&_hott_serial_buffer[0];

word ralenti_gaz;       // seuil ralenti commande des gaz pour chauffage
word coupure_gaz;       // seuil coupure commande des gaz pour chauffage
word pos_armement;      // armement. à la mise sous tension position sécu 
boolean gaz_inverse;    // sens des gaz inversé (ou normal)
word delai;             // délai de temporisation de chauffage bougie
word nb_pales;          // nombre de pales de l'hélice
word courant_dem;       // Courant de chauffage au démarrage
word courant_trn;       // Courant de chauffage "en marche"
word vitesse_gaz;       // Vitesse de déplacement du manche des gaz déclenchant la chauffage de la bougie
                        // en µS pour 20 ms (20ms = période impulsion RX)
word regime_moteur_maxi; // Régime moteur maxi du compte-tours                        
word alarme_moteur;     // temps alarme moteur
word alarme_repet;      // temps de repetition alarme
word alarme_temperature; // temperature d'alarme moteur
word alarme_on;         // alarme on
word langue;            // langue des menus de parametrage

extern word position_gaz;      // Position courante de commande des gaz
extern word trmn_helice_hott;   // tours par minutes hélice
extern byte pwm_affichage_temp; // pour affichage PWM sut Temp 2 Hott

extern word Temps_moteur;  // en s 
extern int Temperature_capteur; // temperature capteur moteur

extern byte bip_alarme;         // son de l'alarme

// For communication
static uint8_t octet1 = 0;  // reception
static uint8_t octet2 = 0;  // reception

// Sauvegardedes parametres en EEPROM 
/*
 !WARNING!
 Writing takes 3.3ms.
 Maximum life of the EEPROM is 100000 writings/readings.
 */
// Adresses des parametres en EEPROM, tous les parametres sont stockes sur 16 bits.  
#define adr_eprom_init 0                 // For the test for 1st time init of the Arduino (First power on)
#define adr_eprom_gaz_inverse 2          // Sens gaz normal '0' ou inversé '1'
#define adr_eprom_ralenti_gaz 4          // Seuil du ralenti
#define adr_eprom_coupure_gaz 6          // Seuil de coupure du moteur / chauffage
#define adr_eprom_pos_armement 8         // Seuil d'armement (sécurité à la mise sous tension
#define adr_eprom_nb_pales 10            // nombre de pales hélice
#define adr_eprom_delai_chauffe 12       // tempo de chauffage
#define adr_eprom_courant_dem 14         // courant moteur à l'arrêt pour démarrage
#define adr_eprom_courant_trn 16         // courant moteur tournant
#define adr_eprom_vitesse_gaz 18         // Vitesse de déplacement du manche des gaz déclenchant la chauffage de la bougie
#define adr_eprom_regime_max 20          // Régime moteur maxi du compte-tours 
#define adr_eprom_alarme_moteur 22       // Temps moteur tournant en secondes pour alarme
#define adr_eprom_alarme_repet 24        // Temps de repetition de l'alarme en secondes
#define adr_eprom_alarme_temperature 26  // Temperature maxi moteur
#define adr_eprom_alarme_on 28           // Activation des alarmes
#define adr_eprom_langue 30              // Langue menu parametrage

// Gestion des mesure de tension
extern Mesure analogique;

GMessage::GMessage(){

}
void GMessage::init(){
  
// Initialisation COM 19200 bauds pour recepteur HOTT GRxx
  Serial.begin (SERIAL_COM_SPEED); 
  
  // ******************************************************
  // Test for 1st time init of the Arduino (First power on)
  int test = read_eprom(adr_eprom_init);
  if (test != 126)
  {
    write_eprom(adr_eprom_init,126);
    write_eprom(adr_eprom_gaz_inverse,0);  // 0 = Normal  1 = Inverse
    write_eprom(adr_eprom_ralenti_gaz,RALENTI_GAZ);
    write_eprom(adr_eprom_coupure_gaz,COUPURE_GAZ);
    write_eprom(adr_eprom_pos_armement,POS_ARMEMENT);
    write_eprom(adr_eprom_nb_pales,NB_PALES);
    write_eprom(adr_eprom_delai_chauffe,DELAI_CHAUFFAGE);
    write_eprom(adr_eprom_courant_dem,COURANT_DEM);
    write_eprom(adr_eprom_courant_trn,COURANT_TRN);
    write_eprom(adr_eprom_vitesse_gaz,VITESSE_GAZ);
    write_eprom(adr_eprom_regime_max,REG_MOT_MAX); 
    write_eprom(adr_eprom_alarme_moteur,ALARME_MOTEUR); 
    write_eprom(adr_eprom_alarme_repet,ALARME_REPET); 
    write_eprom(adr_eprom_alarme_temperature,ALARME_TEMPERATURE);    
    write_eprom(adr_eprom_alarme_on,ALARME_ON); 
    write_eprom(adr_eprom_langue,FRANCAIS);    	
  }
  // Read saved values from EEPROM
  gaz_inverse = read_eprom(adr_eprom_gaz_inverse);        // sens gaz Normal / Inversé  
  ralenti_gaz = read_eprom(adr_eprom_ralenti_gaz);        // seuil ralenti commande des gaz pour chauffage
  coupure_gaz = read_eprom(adr_eprom_coupure_gaz);        // seuil coupure commande des gaz pour chauffage
  pos_armement = read_eprom(adr_eprom_pos_armement);      // armement. à la mise sous tension position sécu 
  nb_pales = read_eprom(adr_eprom_nb_pales);              // nombre de pales entre 1 et 5
  delai = read_eprom(adr_eprom_delai_chauffe);            // délai de chauffage 
  courant_dem = read_eprom(adr_eprom_courant_dem);        // courant de chauffage
  courant_trn = read_eprom(adr_eprom_courant_trn);        // courant de chauffage  
  vitesse_gaz = read_eprom(adr_eprom_vitesse_gaz);        // vitesse de dplacement du manche des gaz
  regime_moteur_maxi = read_eprom(adr_eprom_regime_max);  // Régime moteur maxi du compte-tours  
  alarme_moteur = read_eprom(adr_eprom_alarme_moteur);    // Temps alarme moteur  
  alarme_repet  = read_eprom(adr_eprom_alarme_repet);     // temps de repetition
  alarme_temperature = read_eprom(adr_eprom_alarme_temperature);  // Temperature max moteur
  alarme_on = read_eprom(adr_eprom_alarme_on);            // Activation des alarmes
  langue = read_eprom (adr_eprom_langue);                 // Langue menu parametrage
}

// Lecture d'un mot de 16 bits en EEPROM
uint16_t GMessage::read_eprom(int address){
  return  (uint16_t) EEPROM.read(address) * 256 + EEPROM.read(address+1) ;
}

// Ecriture d'un mot de 16 bits en EEPROM
void GMessage::write_eprom(int address,uint16_t val){
  EEPROM.write(address, val  / 256);
  EEPROM.write(address+1,val % 256 );
}

// Initialisation message GAM (General Air Module)
void GMessage::init_gam_msg(){
  //met tous à Zero, puis on modifie les constantes
  memset(hott_gam_msg, 0, sizeof(struct HOTT_GAM_MSG));   
  hott_gam_msg->start_byte = 0x7c;
  hott_gam_msg->gam_sensor_id = HOTT_TELEMETRY_GAM_SENSOR_ID;
  hott_gam_msg->sensor_id = 0xd0;
  hott_gam_msg->stop_byte = 0x7d;
}

// Emission de la trame
void GMessage::send(int lenght){ 
  uint8_t sum = 0;
  delay(5);
  for(int i = 0; i < lenght-1; i++){
    sum = sum + _hott_serial_buffer[i];
    Serial.write (_hott_serial_buffer[i]);
    delayMicroseconds(HOTTV4_TX_DELAY);

    Serial.read(); // A cause du rebouclage TX <--> RX Lecture de l'octet émis 
  }  
  //Emision du checksum
  Serial.write (sum);
  delayMicroseconds(HOTTV4_TX_DELAY);
  Serial.read(); // A cause du rebouclage TX <--> RX Lecture de l'octet émis 
}

void GMessage::main(){ 
  // 
  static byte page_settings = 1;//numéro de la page de settings à afficher 
    
  if (Serial.available() >= 2) {
    uint8_t octet1 = Serial.read();
    switch (octet1) {
    case HOTT_BINARY_MODE_REQUEST_ID:
      { 
        uint8_t  octet2 = Serial.read();        
        // Demande RX Module =	$80 $XX
        switch (octet2) {
        case HOTT_TELEMETRY_GAM_SENSOR_ID: //0x8D
        {
            // init structure
            init_gam_msg();        
            
            // ******************************************************
            // Trm/min hélice
            setRPM (abs(trmn_helice_hott/10)); // ecriture RPM
            // Tension LIPO ou LIFE
            //setMainVoltage (analogique.Tension_LI ());
            setVoltage1 (analogique.Tension_LI ());
            // Tension NIMH
            setLipo(analogique.Tension_NIMH (), 1);
            // Temperature moteur capteur CTN
            setTemp(Temperature_capteur,1);
            // Commande PWM bougie
            setTemp(pwm_affichage_temp,2);
            setFuelPercent (pwm_affichage_temp);  
            // Temps moteur sur "Voltage 2" en minutes et dixiemes de minutes 
            setVoltage2 ((float) Temps_moteur/60.0);          
            // Gestion Alarme           
            if (alarme_on == 1)
            { //Alarme On, emission du code sonore
              alarme (bip_alarme); 
              bip_alarme = 0;   // annule l'alarme
            }
            else
              alarme(0);        //Pas d'alarme
              
            // altitude 0 et montée 0
            setAltitudeInMeters (0);
            setClimbrate (0);
            setClimbrate_M3 (0);              
                        
            send(sizeof(struct HOTT_GAM_MSG));
            break;
          } //end case GAM*/
        } //end case octet 2
        break;
      }

    case HOTT_TEXT_MODE_REQUEST_ID:
      {      
        uint8_t  octet3 = Serial.read();
        byte id_sensor = (octet3 >> 4);
        byte id_key = octet3 & 0x0f;
        static byte ligne_select = 1 ; // Ligne 1 selectionnée par défaut
        static int8_t ligne_edit = -1 ;
        hott_txt_msg->start_byte = 0x7b;
        hott_txt_msg->esc = 0;
        hott_txt_msg->warning_beeps = 0;
        
        memset((char *)&hott_txt_msg->text, 0x20, HOTT_TEXTMODE_MSG_TEXT_LEN);
        hott_txt_msg->stop_byte = 0x7d;
        
          if (id_sensor == (HOTT_TELEMETRY_GAM_SENSOR_ID & 0x0f)) 
          {
            
            switch (page_settings) { // Pages de parametres GlowHott
              
              case 1:   //PAGE 1 
              {          
                    if (ligne_edit == -1)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP : 
                          ligne_select = min(4,ligne_select + 1); // Ligne 4 maxi                 
                          break;                
                          
                        case HOTT_KEY_DOWN :
                          ligne_select = max(1,ligne_select - 1); //Ligne 1 mini
                          break; 
                          
                        case HOTT_KEY_SET :                      // Sélection de la ligne courante
                          ligne_edit = ligne_select ;
                          break;
                          
                        case HOTT_KEY_LEFT :        // Sortie
                          hott_txt_msg->esc = 0x01;
                          ligne_edit = -1 ;
                          ligne_select = 1;
                          break;  
                                           
                        case HOTT_KEY_RIGHT :      // passage en page 2              
                          page_settings = 2;
                          ligne_edit = -1 ;     
                          ligne_select = 1;                     
                          break;    
                        
                        default : break;
                      }
                    }   
                      
                    //*************************************************
                    //LIGNE 2 SELECTIONNEE = Sens gaz Normal ou inversé
                    else if (ligne_select == 1)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          gaz_inverse = !gaz_inverse;                     
                          break;
                          
                        case HOTT_KEY_DOWN :
                          gaz_inverse = !gaz_inverse;    
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_gaz_inverse,gaz_inverse); // Ecriture en memoire
                          break;
                          
                        default : break;
                      }
                    }          
                    
                    //*************************************************
                    //LIGNE 3 SELECTIONNEE = Seuil de ralenti 
                    else if (ligne_select == 2)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          ralenti_gaz = ralenti_gaz + 5;
                          if (ralenti_gaz > 2100) 
                            ralenti_gaz = 2100;                     
                          break;
                          
                        case HOTT_KEY_DOWN :
                          ralenti_gaz = ralenti_gaz - 5;
                          if (ralenti_gaz < 900) 
                            ralenti_gaz = 900; 
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_ralenti_gaz,ralenti_gaz); // Ecriture en memoire
                          break;
                          
                          
                        default : break;
                      }
                    }                          
                    
                    //*************************************************
                   //LIGNE 4 SELECTIONNEE = Seuil de coupure des gaz 
                    else if (ligne_select == 3)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          coupure_gaz = coupure_gaz + 10;
                          if (coupure_gaz > 2100) 
                            coupure_gaz = 2100;                 
                          break;
                          
                        case HOTT_KEY_DOWN :
                          coupure_gaz = coupure_gaz - 10;
                          if (coupure_gaz < 900) 
                            coupure_gaz = 900;     
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_coupure_gaz,coupure_gaz); // Ecriture en memoire
                          break;

                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;     
                        
                        default : break;
                      }
                    }                                      
                   
                    //*************************************************
                   //LIGNE 5 SELECTIONNEE = Seuil d'armement
                    else if (ligne_select == 4)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          pos_armement = pos_armement + 10;
                          if (pos_armement > 2100) 
                            pos_armement = 2100;                  
                          break;
                          
                        case HOTT_KEY_DOWN :
                          pos_armement = pos_armement - 10;
                          if (pos_armement < 900) 
                            pos_armement = 900;     
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_pos_armement,pos_armement); // Ecriture en memoire
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                               
                          
                        default : break;
                      }
                    }                      

                    //******************************************************************************************
                    // Affichage de la page 1
                    //ligne 0:
                    snprintf((char *)&hott_txt_msg->text[0],21," * Glow Hott V%s *",VERSION);
					
					if (langue == FRANCAIS) {
                      //ligne 1:
                      if (gaz_inverse == false) 
                        snprintf((char *)&hott_txt_msg->text[1],21," Sens gaz     Normal");
                      else 
                        snprintf((char *)&hott_txt_msg->text[1],21," Sens gaz    Inverse");
                      //ligne 2:
                      snprintf((char *)&hott_txt_msg->text[2],21," Ralenti     %4i us",ralenti_gaz);
                      //ligne 3:
                      snprintf((char *)&hott_txt_msg->text[3],21," Coupure gaz %4i us",coupure_gaz);
                      //ligne 4:
                      snprintf((char *)&hott_txt_msg->text[4],21," Armement    %4i us",pos_armement);
                      //ligne 5:
                      snprintf((char *)&hott_txt_msg->text[5],21," - - - - - - - - - -");
                      //ligne 6:
                      snprintf((char *)&hott_txt_msg->text[6],21," Gaz courant %4i us",position_gaz);
					} 
					else {     //if (langue == ANGLAIS) {
                      if (gaz_inverse == false) 
                        snprintf((char *)&hott_txt_msg->text[1],21," Thro. dir.   Normal");
                      else 
                        snprintf((char *)&hott_txt_msg->text[1],21," Thro. dir.  Reverse");
                      //line 2:
                      snprintf((char *)&hott_txt_msg->text[2],21," Thro. idle  %4i us",ralenti_gaz);
                      //line 3:
                      snprintf((char *)&hott_txt_msg->text[3],21," Thro. cut   %4i us",coupure_gaz);
                      //line 4:
                      snprintf((char *)&hott_txt_msg->text[4],21," Arming      %4i us",pos_armement);
                      //line 5:
                      snprintf((char *)&hott_txt_msg->text[5],21," - - - - - - - - - -");
                      //line 6:
                      snprintf((char *)&hott_txt_msg->text[6],21," Thro. pos.  %4i us",position_gaz);										
					}
                    //ligne 7:
                    snprintf((char *)&hott_txt_msg->text[7],21," thyzoon.fr     2015"); 
                    
                    hott_txt_msg->text[ligne_select][0] = '>';
                    _hott_invert_ligne(ligne_edit);
                    break;                    
              }//END PAGE 1
                  
              //-------------------------------------------------------------------------------------------------    
              case 2:             //PAGE 2 
              {          
                    if (ligne_edit == -1)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP : 
                          ligne_select = min(6,ligne_select + 1); // // Ligne 6 maxi                 
                          break;                
                          
                        case HOTT_KEY_DOWN :
                          ligne_select = max(1,ligne_select - 1); // // Ligne 1 mini
                          break; 
                          
                        case HOTT_KEY_SET :   // Sélection de la ligne courante
                          ligne_edit = ligne_select ;
                          break;
                          
                        case HOTT_KEY_LEFT :    // passage en page 1  
                          page_settings = 1;
                          ligne_edit = -1;
                          ligne_select = 1;                           
                          break;
                          
                        case HOTT_KEY_RIGHT :   // passage en page 3              
                          page_settings = 3;
                          ligne_edit = -1 ;     
                          ligne_select = 1;                     
                          break;  
                          
                        default : break;
                      }
                    }   
                      
                    //*************************************************
                    //LIGNE 2 SELECTIONNEE = Courant demarrage (PWM courant bougie)
                    else if (ligne_select == 1)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          courant_dem = courant_dem + 10; 
                          if (courant_dem > 100)
                            courant_dem = 100;                     
                          break;
                          
                        case HOTT_KEY_DOWN :
                          courant_dem = courant_dem - 10; 
                          if (courant_dem <= 50)
                              courant_dem = 50;    
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_courant_dem,courant_dem);
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }
                    }
                    
                    //*************************************************
                    //LIGNE 3 SELECTIONNEE = Courant moteur tournant (PWM courant bougie)
                    else if (ligne_select == 2)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          courant_trn = courant_trn + 10; 
                          if (courant_trn > 100)
                            courant_trn = 100;                     
                          break;
                          
                        case HOTT_KEY_DOWN :
                          courant_trn = courant_trn - 10; 
                          if (courant_trn <= 30)
                              courant_trn = 30;    
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_courant_trn,courant_trn);
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }
                    }
                    
                    //*************************************************
                    //LIGNE 4 SELECTIONNEE = Delai
                    else if (ligne_select == 3)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          delai = delai + 500; 
                          if (delai > 2000)
                            delai = 2000;                     
                          break;
                          
                        case HOTT_KEY_DOWN :
                          delai = delai - 500; 
                          if (delai <= 500)
                              delai = 500;    
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_delai_chauffe,delai);

                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }
                    }                   

                    //*************************************************
                    //LIGNE 5 SELECTIONNEE = nb pales hélice (entre 1 et 5)
                    else if (ligne_select == 4)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          nb_pales = nb_pales + 1; 
                          if (nb_pales > 5)
                            nb_pales = 5;                     
                          break;
                          
                        case HOTT_KEY_DOWN :
                          nb_pales = nb_pales - 1; 
                          if (nb_pales < 1)
                              nb_pales = 1;    
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_nb_pales,nb_pales);
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }
                    } 
                    
                    //*************************************************
                    //LIGNE 6 SELECTIONNEE = vitesse manche des gaz
                    else if (ligne_select == 5)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          vitesse_gaz = vitesse_gaz + 5; 
                          if (vitesse_gaz > 90)
                            vitesse_gaz = 90;                     
                          break;
                          
                        case HOTT_KEY_DOWN :
                          vitesse_gaz = vitesse_gaz - 5; 
                          if (vitesse_gaz < 30)
                              vitesse_gaz = 30;    
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_vitesse_gaz,vitesse_gaz);
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }
                    } 

                    //*************************************************
                    //LIGNE 7 SELECTIONNEE = Regime max. moteur
                    else if (ligne_select == 6)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          regime_moteur_maxi = regime_moteur_maxi + 100; 
                          if (regime_moteur_maxi > 2500)  // 25000 tr/min
                            regime_moteur_maxi = 2500;                     
                          break;
                          
                        case HOTT_KEY_DOWN :
                          regime_moteur_maxi = regime_moteur_maxi - 100; 
                          if (regime_moteur_maxi < 1000)  // 10000 tr/min
                              regime_moteur_maxi = 1000;    
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_regime_max,regime_moteur_maxi); 
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }
                    }                    
                    
                    //******************************************************************************************
                    // Affichage de la page 2
                    //ligne 0:
                    snprintf((char *)&hott_txt_msg->text[0],21," * Glow Hott V%s *",VERSION);
					if (langue == FRANCAIS) {
                      //ligne 1:
                      snprintf((char *)&hott_txt_msg->text[1],21," Courant demarr %3i%%",courant_dem);
                      //ligne 2:
                      snprintf((char *)&hott_txt_msg->text[2],21," Courant moteur %3i%%",courant_trn);
                      //ligne 3:
                      snprintf((char *)&hott_txt_msg->text[3],21," Delai       %4i ms",delai);
                      //ligne 4:
                      snprintf((char *)&hott_txt_msg->text[4],21," Nb pales          %i",nb_pales);
                      //ligne 5:
                      snprintf((char *)&hott_txt_msg->text[5],21," Vitesse Gaz      %i",vitesse_gaz);
                      //ligne 6:
                      snprintf((char *)&hott_txt_msg->text[6],21," Regime Maxi   %i",regime_moteur_maxi*10);
					}
					else {     //if (langue == ANGLAIS) {					
                      //line 1:
                      snprintf((char *)&hott_txt_msg->text[1],21," Start current  %3i%%",courant_dem);
                      //line 2:
                      snprintf((char *)&hott_txt_msg->text[2],21," Run current    %3i%%",courant_trn);
                      //line 3:
                      snprintf((char *)&hott_txt_msg->text[3],21," Delay       %4i ms",delai);
                      //line 4:
                      snprintf((char *)&hott_txt_msg->text[4],21," No. of blades     %i",nb_pales);
                      //line 5:
                      snprintf((char *)&hott_txt_msg->text[5],21," Throt. speed     %i",vitesse_gaz);
                      //line 6:
                      snprintf((char *)&hott_txt_msg->text[6],21," Max revs:     %i",regime_moteur_maxi*10);					
					}
                    //line 7:
                    snprintf((char *)&hott_txt_msg->text[7],21," thyzoon.fr     2015"); 
                    
                    hott_txt_msg->text[ligne_select][0] = '>';
                    _hott_invert_ligne(ligne_edit);
                    break;                    
              }//END PAGE 2   

              //-------------------------------------------------------------------------------------------------    
              case 3://PAGE 3 SETTINGS
              {          
                    if (ligne_edit == -1)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP : 
                          ligne_select = min(5,ligne_select + 1); // // Ligne 5 maxi                 
                          break;                
                          
                        case HOTT_KEY_DOWN :
                          ligne_select = max(1,ligne_select - 1); // // Ligne 1 mini 
                          break; 
                          
                        case HOTT_KEY_SET :   // Sélection de la ligne courante
                          ligne_edit = ligne_select ;
                          break;
                          
                        case HOTT_KEY_LEFT :   // retour Page 2
                          page_settings = 2;
                          ligne_edit = -1;
                          ligne_select = 1;                           
                          break;
                          
                        case HOTT_KEY_RIGHT :  // derniere page                 
                          break;  
                          
                        default : break;
                      }
                    }   
                      
                    //*************************************************
                    //LIGNE 2 SELECTIONNEE = Alarme On / OFF
                    else if (ligne_select == 1)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          if (alarme_on == 0)
                             alarme_on = 1;                  
                          else
                             alarme_on = 0;   
                          break;
                          
                        case HOTT_KEY_DOWN :
                          if (alarme_on == 0)
                             alarme_on = 1;                  
                          else
                             alarme_on = 0;   
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_alarme_on,alarme_on); // Ecriture EEPROM
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;    // sortie de selection 
                          break;                            
                          
                        default : break;
                      }
                    }
                    
                    //*************************************************
                    //LIGNE 3 SELECTIONNEE = Alarme temps moteur tournant 
                    else if (ligne_select == 2)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          alarme_moteur = alarme_moteur + 60;  	// secondes
                          if (alarme_moteur > 1200)   // 20 minutes
                            alarme_moteur = 1200;                     
                          break;
                          
                        case HOTT_KEY_DOWN :
                          alarme_moteur = alarme_moteur - 60; // secondes
                          if (alarme_moteur <= 120)   // 2 minutes
                            alarme_moteur = 120;    
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_alarme_moteur,alarme_moteur); // Ecriture EEPROM
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;          // sortie de selection 
                        break;                            
                          
                        default : break;
                      }
                    }
                    
                    //*************************************************
                    //LIGNE 4 SELECTIONNEE = Delai de repetition de l'alarme temps moteur
                    else if (ligne_select == 3)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          alarme_repet = alarme_repet + 15;   // secondes
                          if (alarme_repet > 60)   // secondes
                            alarme_repet = 60;                     
                          break;
                          
                        case HOTT_KEY_DOWN :
                          alarme_repet = alarme_repet - 15;  // secondes
                          if (alarme_repet <= 0)  
                              alarme_repet = 0;    
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_alarme_repet,alarme_repet);

                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }
                    }                   

                    //*************************************************
                    //LIGNE 5 SELECTIONNEE = Temperature moteur d'alarme (°C)
                    else if (ligne_select == 4)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          alarme_temperature = alarme_temperature + 5;     // °C
                          if (alarme_temperature > 150)
                            alarme_temperature = 150;                     
                          break;
                          
                        case HOTT_KEY_DOWN :
                          alarme_temperature = alarme_temperature - 5;      // °C
                          if (alarme_temperature < 30)
                              alarme_temperature = 30;    
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_alarme_temperature,alarme_temperature);
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;
                        break;                            
                          
                        default : break;
                      }
                    } 
                     
                    //*************************************************
                    //LIGNE 6 SELECTIONNEE = Langue
                    else if (ligne_select == 5)
                    {
                      switch (id_key) {
                        case HOTT_KEY_UP :
                          if (langue == FRANCAIS)
                             langue = ANGLAIS;                  
                          else
                             langue = FRANCAIS;   
                          break;
                          
                        case HOTT_KEY_DOWN :
                          if (langue == FRANCAIS)
                             langue = ANGLAIS;                  
                          else
                             langue = FRANCAIS;   
                          break;
                          
                        case HOTT_KEY_SET :  
                          ligne_edit = -1 ;
                          write_eprom(adr_eprom_langue,langue); // Ecriture EEPROM Langue
                          break;
                          
                        case HOTT_KEY_LEFT : 
                          ligne_edit = -1;    // sortie de selection 
                          break;                            
                          
                        default : break;
                      }
                    }
					
                    //******************************************************************************************
                    // Affichage de la page 3
                    //line 0:
                    snprintf((char *)&hott_txt_msg->text[0],21," * Glow Hott V%s *",VERSION);
					if (langue == FRANCAIS) {
                      //line 1:
                      if (alarme_on == 1)
                        snprintf((char *)&hott_txt_msg->text[1],21," Alarme           ON");
		              else 
                        snprintf((char *)&hott_txt_msg->text[1],21," Alarme          OFF");
                      //line 2:
                      snprintf((char *)&hott_txt_msg->text[2],21," Temps moteur %2i min",alarme_moteur/60);
                      //line 3:
                      snprintf((char *)&hott_txt_msg->text[3],21," Repetition     %2i s",alarme_repet);
                      //line 4:
                      snprintf((char *)&hott_txt_msg->text[4],21," Temperature %3i Deg",alarme_temperature);
                      //line 5:
                      snprintf((char *)&hott_txt_msg->text[5],21," Langue     Francais");
                      //line 6:
                      snprintf((char *)&hott_txt_msg->text[6],21," Bons vols !");
					}  
					else {     //if (langue == ANGLAIS) {	  
                      if (alarme_on == 1)
                        snprintf((char *)&hott_txt_msg->text[1],21," Alarm            ON");
		              else 
                        snprintf((char *)&hott_txt_msg->text[1],21," Alarm           OFF");
                      //line 2:
                      snprintf((char *)&hott_txt_msg->text[2],21," Engine run   %2i min",alarme_moteur/60);
                      //line 3:
                      snprintf((char *)&hott_txt_msg->text[3],21," Repeat         %2i s",alarme_repet);
                      //line 4:
                      snprintf((char *)&hott_txt_msg->text[4],21," Temperature %3i Deg",alarme_temperature);	
                      //line 5:
					  snprintf((char *)&hott_txt_msg->text[5],21," Language    English");
                      //line 6:		
					  snprintf((char *)&hott_txt_msg->text[6],21," Good flying !");
		  			}  
                    //line 7:
                    snprintf((char *)&hott_txt_msg->text[7],21," thyzoon.fr     2015"); 
                    
                    hott_txt_msg->text[ligne_select][0] = '>';
                    _hott_invert_ligne(ligne_edit);
                    break;                    
              }//END PAGE 3                    
                  
                  default://PAGE 
                  {
                    break;
                  }            
                 
            }//END SETTINGS 
       
          } // END IF 
          
          //********************************************************************************************************************

          else if(id_sensor == (HOTT_TELEMETRY_GEA_SENSOR_ID & 0x0f)) {
            snprintf((char *)&hott_txt_msg->text[0],21,"Module EAM");
            snprintf((char *)&hott_txt_msg->text[1],21,"----------");
            snprintf((char *)&hott_txt_msg->text[2],21,"Selectionner :");
            snprintf((char *)&hott_txt_msg->text[3],21," GENERAL");
          }
          else if(id_sensor == (HOTT_TELEMETRY_VARIO_SENSOR_ID & 0x0f)) {         
            snprintf((char *)&hott_txt_msg->text[0],21,"Module VARIO");
            snprintf((char *)&hott_txt_msg->text[1],21,"------------");
            snprintf((char *)&hott_txt_msg->text[2],21,"Selectionner :");
            snprintf((char *)&hott_txt_msg->text[3],21," GENERAL");
          }
          else {
            snprintf((char *)&hott_txt_msg->text[0],21,"Module inconnu");
            snprintf((char *)&hott_txt_msg->text[1],21,"--------------");
            snprintf((char *)&hott_txt_msg->text[2],21,"Selectionner :");
            snprintf((char *)&hott_txt_msg->text[3],21," GENERAL");
          }
        //}
        // Emission en mode texte
        _hott_send_text_msg();
        break;
      }
    }	
  }
}

//////////////////////////////////////////////////
// Tension principale (resolution 0.1V)
void GMessage::setMainVoltage(float tension){
  hott_gam_msg->main_voltage = (uint16_t) (tension * 10);
}

// Batterie 1 (resolution 0.1V)
void GMessage::setVoltage1(float volt){
  hott_gam_msg->Battery1 =  (uint16_t) (volt * 10); 
}

// Batterie 2 (resolution 0.1V)
void GMessage::setVoltage2(float volt){
  hott_gam_msg->Battery2 = (uint16_t) (volt * 10); 
}

// Element Lipo n (resolution 0.02V)
void GMessage::setLipo(float volt, int lipo){

  if (lipo >= 1 and lipo <= 6)
  {
    lipo--;
    hott_gam_msg->cell[lipo] =  (uint16_t) 100 * volt / 2 ; 
    
    if (volt/2 <= hott_gam_msg->min_cell_volt || hott_gam_msg->min_cell_volt ==0)
    {
      hott_gam_msg->min_cell_volt = (uint16_t) 100 * volt/2 ;
      hott_gam_msg->min_cell_volt_num = lipo;
    }
  }
}

// Altitude relative en metres : -500 / +9999 [m]
void GMessage::setAltitudeInMeters(uint16_t alti){
  hott_gam_msg->altitude = alti + 500;
}

// Temperature : -20 a +235° C 
// 
void GMessage::setTemp(int temp, int capteur){
  if(temp < -20)
    temp = -20;
  else if(temp > 234)
    temp = 235;
  if (capteur < 1)
    capteur = 1;
  if (capteur > 2)
    capteur = 2; 
  if (capteur == 1)
    hott_gam_msg->temperature1 = temp + 20;
  else if (capteur == 2)
    hott_gam_msg->temperature2 = temp + 20;
}

// Niveau reservoir : 0%, 25%, 50%, 75%, 100%  
// 
void GMessage::setFuelPercent(uint8_t pourcent){
  if(pourcent > 100)
    pourcent = 100;
  else if(pourcent < 0)
    pourcent = 0;
  hott_gam_msg->fuel_procent = pourcent ;
}

// niveau reservoir en ml : 0 a 65535 ml
// 
void GMessage::setFuelMilliliters(uint16_t ml){
  hott_gam_msg->fuel_ml = ml ;
}

// Rotation : 0 a 655350 tr/min
// 

void GMessage::setRPM(uint16_t rpm){
  hott_gam_msg->rpm = rpm ;
} 

// Set climbrates
void GMessage::setClimbrate(uint16_t climb_L){
  hott_gam_msg->climbrate_L =  climb_L+30000;
}

void GMessage::setClimbrate_M3(int climb3s){
  hott_gam_msg->climbrate3s =  climb3s+120;
}

// Set speed
void GMessage::setSpeed(float speed){
  hott_gam_msg->climbrate3s =  (int) speed;
}


//
// Alarme sonore sur radio
//
void GMessage::alarme (uint8_t son){
  hott_gam_msg->warning_beeps = son;
}

void GMessage::_hott_send_text_msg() {
  for(byte *_hott_msg_ptr = hott_txt_msg->text[0]; _hott_msg_ptr < &hott_txt_msg->stop_byte ; _hott_msg_ptr++){
    if (*_hott_msg_ptr == 0)
      *_hott_msg_ptr = 0x20;
  }  
  //_hott_send_msg(_hott_serial_buffer, sizeof(struct HOTT_TEXTMODE_MSG));
  send(sizeof(struct HOTT_TEXTMODE_MSG));
}

char * GMessage::_hott_invert_all_chars(char *str) {
  return _hott_invert_chars(str, 0);
}

char * GMessage::_hott_invert_chars(char *str, int cnt) {
  if(str == 0) return str;
  int len = strlen(str);
  if((len < cnt)  && cnt > 0) len = cnt;
  for(int i=0; i< len; i++) {
    str[i] = (byte)(0x80 + (byte)str[i]);
  }
  return str;
}

void GMessage::_hott_invert_ligne(int ligne) {
  if (ligne>= 0 && ligne<= 7)
    for(int i=0; i< 21; i++) {
      if (hott_txt_msg->text[ligne][i] == 0)   //inversion du caratère null (fin de string)
        hott_txt_msg->text[ligne][i] = (byte)(0x80 + 0x20);
      else
        hott_txt_msg->text[ligne][i] = (byte)(0x80 + (byte)hott_txt_msg->text[ligne][i]);
    }
}



