//
//  Mesure des tensions et température
//
//  www.thyzoon.fr
//  2015
//

#include "Arduino.h"
#include "Mesure.h"
#include <math.h>

Mesure::Mesure () {
}

// Initialisation CAN et entrées
void Mesure::init(){
   analogReference(DEFAULT); // reference alim. 5V
   
   // Configuration des entrées Analogiques.
   digitalWrite(A0, LOW); 
   pinMode(A0, INPUT);
   digitalWrite(A1, LOW); 
   pinMode(A1, INPUT);
   digitalWrite(A2, LOW); 
   pinMode(A2, INPUT);
}

//
// Mesure de la tension de l'accumulateur NIMH sur A0
float Mesure::Tension_NIMH () {
  float nimh;

  nimh = lecture_can (A0) * VREF / RESOLUTION;
    
  return nimh;
}

// Mesure de la tension de l'accumulateur LIPO 2S ou LIFE 2S sur A2
float Mesure::Tension_LI () {
  float li;
  
  li = lecture_can (A2) * VREF / RESOLUTION;
  li = (li / R9) * (R9 + R8);
  
  return li;  
}

// Mesure de la temperature (CTN) sur A1 
int Mesure::Temperature () {
  float temperature;
  float tension;
  float courant;
  float rctn;
  int temp;
  
  tension = lecture_can (A1) * VREF / RESOLUTION;       // tension lue par le CAN voie A1 sur CTN
  courant = (5 - tension) / R6;                         // courant dans CTN
  rctn = tension / courant;                             // Valeur resistance CTN
  temperature = B / log (rctn / RINF) + ZERO_KELVIN;    // Calcul temperature en °C
  temperature = temperature - (tension * courant ) / K; // correction : auto échauffement
  temp = round (temperature);
  
  return temp; 
}  

// Mesure tension sur convertisseur A/N 
uint16_t Mesure::lecture_can (uint8_t pin){
  uint16_t val = analogRead(pin); 
  return val;
}
